@@ -69,7 +69,8 @@ check.and.make.unique = function(v,ref=NULL,label="entries",ref.label="reference
 
   if (any(is.na(v) | v=="")) {
     if (is.null(ref)) stop(sprintf("When %s are empty, %s must be provided!",label,ref.label))
-    fun(sprintf("Some %s are empty (n=%d, e.g. %s), replacing by %s ids!",label,sum(is.na(v) | v==""),paste(sample(head(ref[is.na(v) | v==""])),collapse=","),ref.label),call. = FALSE,immediate. = TRUE)
+    howmany=if (sum(is.na(v)|v=="")==length(v)) "All" else "Some"
+    fun(sprintf("%s %s are empty (n=%d, e.g. %s), replacing by %s ids!",howmany,label,sum(is.na(v) | v==""),paste(sample(head(ref[is.na(v) | v==""])),collapse=","),ref.label),call. = FALSE,immediate. = TRUE)
     v[is.na(v) | v==""]=ref[is.na(v) | v==""]
   }
 
@@ -425,6 +426,8 @@ ReadCounts=function(file, design=c(Design$Condition,Design$Replicate),classify.g
   }
 
   checknames=function(a,b){
+    if (nrow(a)!=nrow(b)) stop("Number of rows do not match!")
+    if (ncol(a)!=ncol(b)) stop("Number of columns do not match!")
     if (!all(colnames(a)==colnames(b))) stop("Column names do not match!")
     if (!all(rownames(a)==rownames(b))) stop("Row names do not match!")
 
@@ -519,24 +522,34 @@ ReadCounts=function(file, design=c(Design$Condition,Design$Replicate),classify.g
 
 
 GetTableQC=function(data,name,stop.if.not.exist=TRUE) {
-  fn=paste0(data$prefix,".",name,".tsv.gz")
-  if (!file.exists(fn)) {
-    fn2=paste0(data$prefix,".",name,".tsv")
-    if (!file.exists(fn2)) {
-      fn3=paste0(data$prefix,".",name)
-      if (!file.exists(fn3)) {
-        if (stop.if.not.exist) stop(paste0("Cannot find QC table ",fn," or ",fn2))
-        else warning(paste0("Cannot find QC table ",fn," or ",fn2))
-      }
-      fn=fn3
-    } else {
-      fn = fn2
-    }
+  ll=try.file(paste0(data$prefix,".",name),possible.suffixes = c(".tsv.gz",".tsv",""),stop.if.not.exist=stop.if.not.exist)
+  if (is.null(ll)) {
+    warning(paste0("Cannot find QC table ",name))
+    return(NULL)
   }
-  if (!file.exists(fn)) return(NULL)
-
   header = !name %in% c("clip","strandness")
-  read.tsv(fn,header=header)
+  re=read.tsv(ll$file,header=header)
+  ll$callback()
+  return(re)
+
+#  fn=paste0(data$prefix,".",name,".tsv.gz")
+#  if (!file.exists(fn)) {
+#    fn2=paste0(data$prefix,".",name,".tsv")
+#    if (!file.exists(fn2)) {
+#      fn3=paste0(data$prefix,".",name)
+#      if (!file.exists(fn3)) {
+#        if (stop.if.not.exist) stop(paste0("Cannot find QC table ",fn," or ",fn2))
+#        else warning(paste0("Cannot find QC table ",fn," or ",fn2))
+#      }
+#      fn=fn3
+#    } else {
+#      fn = fn2
+#    }
+#  }
+#  if (!file.exists(fn)) return(NULL)
+#
+#  header = !name %in% c("clip","strandness")
+#  read.tsv(fn,header=header)
 }
 
 
@@ -799,11 +812,13 @@ ReadNewTotal=function(genes, cells, new.matrix, total.matrix, detection.rate=1,v
   re
 }
 
-try.file = function(prefix, possible.suffixes=c("",".tsv",".tsv.gz",".targets/data.tsv.gz"),verbose=FALSE) {
+try.file = function(prefix, possible.suffixes=c("",".tsv",".tsv.gz",".targets/data.tsv.gz"),verbose=FALSE, stop.if.not.exist=TRUE) {
   do.callback=function() {}
 
+  cut.suffix=function(p) {for (s in possible.suffixes) p=gsub(paste0(s,"$"),"",p); p}
+
   for (suffix in possible.suffixes) {
-    if (file.exists(paste0(prefix,suffix))) return(list(file=paste0(prefix,suffix),prefix=prefix,callback=do.callback))
+    if (file.exists(paste0(prefix,suffix))) return(list(file=paste0(prefix,suffix),prefix=cut.suffix(prefix),callback=do.callback))
   }
 
 
@@ -818,19 +833,25 @@ try.file = function(prefix, possible.suffixes=c("",".tsv",".tsv.gz",".targets/da
 
 
     if (!is.null(url)) {
-      file <- tempfile()
+      fn=gsub(".*/","",gsub("\\?.*","",url))
+      fn1 = gsub("\\..*","",fn)
+      ext=substr(fn,nchar(fn1)+1,nchar(fn))
+      file <- tempfile(pattern = fn1,fileext = ext)
+
       if (verbose) cat(sprintf("Downloading file (url: %s, destination: %s) ...\n",url,file))
       download.file(url, file, quiet=!verbose)
       do.callback=function() {
         if (verbose) cat("Deleting temporary file...\n")
         unlink(file)
       }
-      return(list(file=file,prefix=prefix,callback=do.callback))
+      prefix=gsub("\\?.*","",prefix) # cut ?x=y extensions
+      return(list(file=file,prefix=cut.suffix(prefix),callback=do.callback))
     }
   } else {
-    stop("File not found; If you want to access non-local files directly, please install the RCurl package!")
+    if (stop.if.not.exist) stop("File not found; If you want to access non-local files directly, please install the RCurl package!")
   }
-  stop("File not found!")
+  if (stop.if.not.exist) stop("File not found!")
+  return(NULL)
 }
 
 read.grand.internal=function(prefix, design=c(Design$Condition,Design$Replicate),
@@ -966,7 +987,9 @@ read.grand.internal=function(prefix, design=c(Design$Condition,Design$Replicate)
   re=list()
   for (n in names(slots)) {
     cols=intersect(paste0(conds," ",n),names(data))
-    re[[slots[n]]]=tomat(data[,cols],data$Gene,cols)
+    a=tomat(data[,cols],data$Gene,cols)
+    if (ncol(a)==0 && n=="Conversions") stop("Columns for conversions are not available in GRAND-SLAM output; rerun gedi -e Slam with parameter -full!")
+    re[[slots[n]]]=a
   }
   #re$count=tomat(data[,count],data$Gene,names(data)[count])
   ##	re$tpm=comp.tpm(re$count,gene.info$Length)
@@ -993,12 +1016,14 @@ read.grand.internal=function(prefix, design=c(Design$Condition,Design$Replicate)
     re$beta=correctmat(re$beta)
   }
 
-  checknames=function(a){
-    if (!all(colnames(a)==coldata$Name)) stop("Column names do not match!")
-    if (!all(rownames(a)==gene.info$Gene)) stop("Row names do not match!")
+  checknames=function(n,a){
+    if (nrow(a)!=nrow(gene.info)) stop(sprintf("Number of rows do not match for %s!",n))
+    if (ncol(a)!=nrow(coldata)) stop(sprintf("Number of columns do not match for %s!",n))
+    if (!all(colnames(a)==rownames(coldata))) stop(sprintf("Column names do not match for %s!",n))
+    if (!all(rownames(a)==gene.info$Gene)) stop(sprintf("Row names do not match for %s!",n))
   }
 
-  for (slot in slots) checknames(re[[slot]])
+  for (slotname in names(re)) checknames(slotname,re[[slotname]])
 
   coldata$no4sU=no4sU.cols
 
