@@ -3,16 +3,19 @@
 read.tsv=function(t,verbose=FALSE,stringsAsFactors=FALSE,...) {
 
   readit=function(file,...)
-    if (requireNamespace("data.table",quietly = TRUE) && summary(file(file))$class!="gzfile") {
+    if (requireNamespace("data.table",quietly = TRUE) && !grepl("\\.gz$",file)) {
       as.data.frame(data.table::fread(file = file,stringsAsFactors=stringsAsFactors,check.names=FALSE,data.table = FALSE,...))
     } else {
       read.delim(file = file,stringsAsFactors=stringsAsFactors,check.names=FALSE,...)
     }
 
   if (suppressWarnings(requireNamespace("RCurl",quietly = TRUE)) && RCurl::url.exists(t)) {
-    file <- tempfile()
-    if (verbose) cat("Downloading file...\n")
-    download.file(url, file, quiet=!verbose)
+    fn=gsub(".*/","",gsub("\\?.*","",t))
+    fn1 = gsub("\\..*","",fn)
+    ext=substr(fn,nchar(fn1)+1,nchar(fn))
+    file <- tempfile(pattern = fn1,fileext = ext)
+    if (verbose) cat(sprintf("Downloading file to %s...\n",file))
+    download.file(t, file, quiet=!verbose)
     if (verbose) cat("Reading file...\n")
     t=readit(file,...)
     if (verbose) cat("Deleting temporary file...\n")
@@ -21,7 +24,6 @@ read.tsv=function(t,verbose=FALSE,stringsAsFactors=FALSE,...) {
     if (verbose) cat("Reading file...\n")
     t=readit(t,...)
   }
-
   if (stringsAsFactors==TRUE) t=as.data.frame(lapply(t,function(c) if (is.character(c)) factor(c,levels=unique(c)) else c),check.names=FALSE)
   t
 }
